$! PROCESS_ABI_FILES.COM
$! 09-AUG-2000, David Mathog, Biology Division, Caltech
$!
$! This procedure waits around in a batch queue and periodically
$! checks for the existence of a flag file.  Once it sees that it
$! runs through a directory structure looking for directories that
$! contain only *.BIN or *.AB1 files.  If it finds one, it makes
$! a list of such files, then processes it through PHRED to call
$! the bases and produce a .PHD file.  Then it uses ABIPRINTOUT
$! to make a printout of the traces.
$!
$! If the name of the flag file is RESTART then it reuses the existing
$! the_do_list file.
$!
$!
$!  Parameters used in this procedure
$!
$! DNA_PICKUP, rooted logical
$! TEMP.PS  temporary file to hold postscript, deleted after printed
$! TEMP.PHD temporary file to hold phred output, deleted after abiprintout uses it
$! TEMP.SEQ temporary file generated by phred, deleted after copied to target location
$! FORPHRED.TXT  temporary file that holds one ABI file to convert for phred
$! 
$ define TESTDIR DNA_PICKUP:[AAAPROCESS.TESTDIR]
$ define WORKDIR DNA_PICKUP:[AAAPROCESS.WORKDIR]
$ PCOMMAND      = "PRINT/QUEUE=DNACOLOR_HP/parameter=(data_type=postscript)/noflag/delete/noidentify"
$ THE_DO_LIST   = "THE_DO_LIST.TXT"
$ postscript colorscript-100 temp.ps nobanner
$ workqueue == "PHREDWORK"
$!
$!  Each job resubmits itself so that if it dies for any reason, there will
$!  be one waiting to take over.
$!
$  submit/que=phredwork/log=workdir:/name=ABIPROC gcgexsource:PROCESS_ABI_FILES.COM
$!
$ set default WORKDIR
$ time = f$time() 
$ if(f$search("WORKDIR:PROCESSING.LOG") .eqs. "")
$ then
$   open/write lfil: WORKDIR:PROCESSING.LOG
$   write lfil: "Daemon start at:    ''time'"
$   close lfil:
$ else
$   open/append/share=read lfil: WORKDIR:PROCESSING.LOG
$   write lfil: "Daemon restart at:  ''time'"
$   close lfil:
$ endif
$!
$! wait for the trigger file to appear, the dummy search of
$! workdir is needed so that new files that appear in TESTDIR
$! will be picked up, otherwise once f$search returns ""
$! it will always do so.
$!
$ WAITHERE:
$  file = f$search("WORKDIR:*.*")
$  file = f$search("TESTDIR:*.*")
$  if(file .eqs. "")
$  then
$    wait 00:01:00.00
$    goto waithere
$  endif
$!
$! allow an extra 10 seconds so that the FTP drop of the file
$! will be sure to complete before we try to delete this file.
$!
$ wait 00:00:10.00
$ delete/nolog TESTDIR:*.*;*
$!
$! obtain the time of the last file processing
$!
$ time=f$time()
$ last_time=""
$ open/error=nolast ifil: WORKDIR:lastphred.txt
$   read/end=badlast/error=badlast ifil: last_time
$ close ifil:
$ goto nolast
$!
$ badlast:
$   last_time=""
$   close ifil:
$ nolast:
$ delete/nolog WORKDIR:lastphred.txt;*
$ open/write ofil: WORKDIR:lastphred.txt
$   write ofil: time
$ close ofil:
$ starttime = time
$!
$!  List all of the files of the appropriate types deposited on the system 
$!  since the last processing
$!
$ if(f$parse(file,,,"NAME") .nes. "RESTART")
$ then
$   dir/out='the_do_list'/since="''last_time'"/notrail/nototal/nogrand/nohead dna_pickup:[000000...]*.bin,*.ab1
$ endif
$!
$ open/error=badlist ifil: 'the_do_list'
$!
$ badcount = 0
$ count = 0
$ mainloop:
$   read/end=done/error=done ifil: string
$   count = count + 1
$   string = string - "000000."
$   sver_type = f$parse(string,,,"TYPE") + f$parse(string,,,"VERSION")
$   targetroot= string - sver_type
$   targetps  = targetroot + ".PS"
$   targetseq = targetroot + ".SEQ"
$   username  = f$element(0,".",string)
$   username  = f$element(1,"[",username)
$   fsize= f$file_attributes(string,"EOF")
$   if(fsize .lt. 210)
$   then
$      write sys$output "Trace file too small to process: ",string
$      delete/nolog 'string'
$      badcount = badcount + 1
$!
$! if it's really small just ignore it, if it's big enough to be
$! a bad trace file mark it as such
$!
$      if(fsize .gt. 20)
$      then
$        targetbad = targetroot + ".BAD__NO_TRACE"
$        create 'targetbad'
This file did not contain trace information.
$      endif
$!
$      goto mainloop
$   endif
$!
$! generate phred and sequence files for this one ABI file
$!
$   define/user sys$output forphred.txt
$   v2u 'string' noversion
$   phred -if forphred.txt -p temp.phd -s temp.seq
$!
$!  if phred screws up it will still create temp.seq, but it will contain only one line
$!  (the > line).  It must be able to read at least some sequence for 
$!  abiprintout to have a chance
$!
$   open/read/error=badphd jfil: temp.seq
$   read/end=badphd2/error=badphd2 jfil: tstring
$   read/end=badphd2/error=badphd2 jfil: tstring
$   close jfil:
$!
$   abiprintout/infile='string'/phd=temp.phd/page=1/pnt=1300/font=0/note="''username'"/default
$   if (.not. $status)
$   then
$      write sys$output "ABIPRINTOUT could not process: ",string
$      badcount = badcount + 1
$      goto cleanup
$   endif
$!
$   rename temp.ps  'targetps'
$   fts=f$search(targetseq) 
$   if(fts .nes. "")then delete/nolog 'fts'
$   copy  temp.seq 'targetseq'
$   pcommand 'targetps'
$   goto cleanup
$$!
$ badphd2:
$   close jfil:
$ badphd:
$   write sys$output "Phred could not process: ",string
$   badcount = badcount + 1
$ cleanup:
$   delete forphred.txt;*
$   delete temp.phd;*
$   delete temp.seq;*
$!  pcommand will delete targetps after it prints it
$   goto mainloop
$!
$ done:
$   close ifil:
$   delete 'the_do_list';
$ badlist:
$   endtime=f$time()
$   open/append/share=read lfil: WORKDIR:PROCESSING.LOG
$     write lfil: "Start:  ''starttime' End: ''endtime' Files: ''count' Bad Files: ''badcount'"
$   close lfil:
$   goto waithere
